#!/usr/bin/perl -w

# Converts a rate category file for the Parisomony like family size model
# Usage: create.pl FILE

my %freq;
open(FREQ,$ARGV[1]);
while (my $line = <FREQ>)
{
	my @split = split(/\t/,$line);
	chomp($split[1]);
	$freq{$split[0]} = $split[1];
}
close(FREQ);


open(RC,">" . $ARGV[0]);

my @dna = qw(A C T G);
my @codon;

for my $d1 (@dna)
{
	for my $d2 (@dna)
	{
		for my $d3 (@dna)
		{
			push(@codon,$d1.$d2.$d3);
		}
	}
}


my $states;

my $first = 1;
for my $c (@codon)
{
	if ($first)
	{
		$states = $c;
		$first = 0;
	}
	else
	{
		$states = $states . "\t" . $c;
	}
}

print RC "64\n";
print RC "\n";
print RC $states . "\n";
print RC "\n";

for my $c1 (@codon)
{
	my $row;
	$first = 1;
	for my $c2 (@codon)
	{
		$rate = getRate($c1, $c2, $ARGV[2]);
		if (($c1 ne $c2) && ($rate ne "0.0") && ($rate ne "1.0"))
		{
			$rate = $rate . "*" . $freq{$c2};
		}
		if ($rate eq "1.0")
		{
			$rate = $freq{$c2};
		}
		if ($first)
		{
			$row = $rate;
			$first = 0;
		}
		else
		{
			$row = $row . "\t" . $rate;
		}
	}
	
	print RC $row . "\n";
}

print RC "\n";

$first = 1;
for my $c (@codon)
{
	if ($first)
	{
		print RC $freq{$c};
		$first = 0;
	}
	else
	{
		print RC  "\t" . $freq{$c};
	}
}

close (RC);

sub getRate
{
	my %map = (
	'GCT' => 'A', 'GCC' => 'A', 'GCA' => 'A', 'GCG' => 'A', 'TTA' => 'L',
	'TTG' => 'L', 'CTT' => 'L', 'CTC' => 'L', 'CTA' => 'L', 'CTG' => 'L',
	'CGT' => 'R', 'CGC' => 'R', 'CGA' => 'R', 'CGG' => 'R', 'AGA' => 'R',
	'AGG' => 'R', 'AAA' => 'K', 'AAG' => 'K', 'AAT' => 'N', 'AAC' => 'N',
	'ATG' => 'M', 'GAT' => 'D', 'GAC' => 'D', 'TTT' => 'F', 'TTC' => 'F',
	'TGT' => 'C', 'TGC' => 'C', 'CCT' => 'P', 'CCC' => 'P', 'CCA' => 'P',
	'CCG' => 'P', 'CAA' => 'Q', 'CAG' => 'Q', 'TCT' => 'S', 'TCC' => 'S',
	'TCA' => 'S', 'TCG' => 'S', 'AGT' => 'S', 'AGC' => 'S', 'GAA' => 'E',
	'GAG' => 'E', 'ACT' => 'T', 'ACC' => 'T', 'ACA' => 'T', 'ACG' => 'T',
	'GGT' => 'G', 'GGC' => 'G', 'GGA' => 'G', 'GGG' => 'G', 'TGG' => 'W',
	'CAT' => 'H', 'CAC' => 'H', 'TAT' => 'Y', 'TAC' => 'Y', 'ATT' => 'I',
	'ATC' => 'I', 'ATA' => 'I', 'GTT' => 'V', 'GTC' => 'V', 'GTA' => 'V',
	'GTG' => 'V', 'TAA' => '*', 'TAG' => '*', 'TGA' => '*');

	$c1 = $_[0];
	$c2 = $_[1];
	$o = $_[2];
	
	if (($map{$c1} eq '*') || ($map{$c2} eq '*'))
	{
		return "0.0";
	}
	
	my $c = 0;
	my $change1;
	my $change2;
	for (my $i = 0; $i < 3; $i++)
	{
		$cc1 = substr($c1,$i,1);
		$cc2 = substr($c2,$i,1);
		if ($cc1 ne $cc2)
		{
			$change1 = $cc1;
			$change2 = $cc2;
			$c++;
		}
	}
	if ($c == 0)
	{
		return "-";
	}
	if ($c > 1)
	{
		return "0.0";
	}
	
	if (($change1 eq 'A' && $change2 eq 'G')
		|| ($change1 eq 'G' && $change2 eq 'A')
		|| ($change1 eq 'C' && $change2 eq 'T')
		|| ($change1 eq 'T' && $change2 eq 'C'))
	{
		if ($map{$c1} eq $map{$c2})
		{
			return "k";
		}
		else
		{
			return "k*" . $o;
		}
	}
	else
	{
		if ($map{$c1} eq $map{$c2})
		{
			return "1.0";
		}
		else
		{
			return $o;
		}
	}
}